@@ -14,12 +14,22 @@ SMOOTH_ODDS_EPS = 5e-3
 
 class ForecastEvent(BaseModel):
     """Individual forecast from a user for a specific problem."""
-    forecast_id: str = Field(description="The id of the forecast")
+    forecast_id: str = Field(description="The unique identifier of the forecast")
     problem_id: str = Field(description="The id of the problem")
     username: str = Field(description="The user name/id of the forecaster")
     timestamp: datetime = Field(description="The timestamp of the forecast")
     probs: List[float] = Field(description="The forecasted probabilities for each option")
     unnormalized_probs: Optional[List[float]] = Field(default=None, description="The unnormalized forecasted probabilities for each option")
+    weight: float = Field(description="The weight of the forecast. This is used to weight the forecast in scoring/ranking. Default to 1.", default=1.0)
+    odds: Optional[List[float]] = Field(None, description="The odds for each option")
+    no_odds: Optional[List[float]] = Field(None, description="The odds for each option to not realize")
+
+    @field_validator('weight')
+    def validate_weight(cls, v):
+        """Validate that weight is non-negative."""
+        if v < 0:
+            raise ValueError("Weight must be non-negative")
+        return v
 
     @field_validator('probs')
     def validate_probabilities(cls, v):
@@ -52,6 +62,34 @@ class ForecastEvent(BaseModel):
      
         return v
 
+    @field_validator('odds')
+    def validate_odds(cls, v, info):
+        """Validate that odds match the number of probabilities if provided."""
+        if v is not None and info.data and 'probs' in info.data:
+            if len(v) != len(info.data['probs']):
+                raise ValueError(f"Number of odds ({len(v)}) must match number of probabilities ({len(info.data['probs'])})")
+
+        # check that odds each has to be in [0, 1]
+        if v is not None and not all(0 <= p <= 1 for p in v):
+            raise ValueError("All odds (implied probabilities) must be in [0, 1]")
+        return v
+
+    @model_validator(mode='after')
+    def smooth_odds(self):
+        """Smooth the odds to not be too close to 0 or 1.
+        """
+        if self.odds is not None:
+            self.odds = [max(SMOOTH_ODDS_EPS, min(1 - SMOOTH_ODDS_EPS, odd)) for odd in self.odds]
+
+        if self.no_odds is not None:
+            self.no_odds = [max(SMOOTH_ODDS_EPS, min(1 - SMOOTH_ODDS_EPS, odd)) for odd in self.no_odds]
+        return self
+
+
+class ProphetArenaForecastEvent(ForecastEvent):
+    """Specialized forecast event for Prophet Arena."""
+    submission_id: str = Field(description="The id of the submission batch. Note this might not be unique across different forecasters.")
+
 
 class ForecastProblem(BaseModel):
     """A prediction problem with multiple options and forecasts."""
@@ -63,8 +101,6 @@ class ForecastProblem(BaseModel):
     end_time: datetime = Field(description="The end time of the problem")
     num_forecasters: int = Field(description="The number of forecasters")
     url: Optional[str] = Field(None, description="The URL of the problem")
-    odds: Optional[List[float]] = Field(None, description="The odds for each option")
-    no_odds: Optional[List[float]] = Field(None, description="The odds for each option to not realize")
     category: Optional[str] = Field(None, description="The category of the problem")
 
     @field_validator('correct_option_idx')
@@ -83,7 +119,7 @@ class ForecastProblem(BaseModel):
 
     @field_validator('forecasts')
     def validate_forecasts(cls, v, info):
-        """Validate that all forecasts have correct number of probabilities."""
+        """Validate that all forecasts have (1) correct number of probabilities, (2) unique `forecast_id`."""
         if info.data and 'options' in info.data:
             expected_length = len(info.data['options'])
             for forecast in v:
@@ -92,40 +128,26 @@ class ForecastProblem(BaseModel):
                         f"Forecast by {forecast.username} has {len(forecast.probs)} probabilities, "
                         f"expected {expected_length}"
                     )
+
+        forecast_id_set = set()
+        for forecast in v:
+            if forecast.forecast_id in forecast_id_set:
+                raise ValueError(f"Forecast by {forecast.username} has duplicate `forecast_id`: {forecast.forecast_id}")
+            forecast_id_set.add(forecast.forecast_id)
+        
         return v
 
-    @field_validator('odds')
-    def validate_odds(cls, v, info):
-        """Validate that odds match the number of options if provided."""
-        if v is not None and info.data and 'options' in info.data:
-            if len(v) != len(info.data['options']):
-                raise ValueError(f"Number of odds ({len(v)}) must match number of options ({len(info.data['options'])})")
-
-        # check that odds each has to be in [0, 1]
-        if v is not None and not all(0 <= p <= 1 for p in v):
-            raise ValueError("All odds (implied probabilities) must be in [0, 1]")
-        return v
-
-    @model_validator(mode='after')
-    def smooth_odds(self):
-        """Smooth the odds to not be too close to 0 or 1.
-        """
-        if self.odds is not None:
-            self.odds = [max(SMOOTH_ODDS_EPS, min(1 - SMOOTH_ODDS_EPS, odd)) for odd in self.odds]
-
-        if self.no_odds is not None:
-            self.no_odds = [max(SMOOTH_ODDS_EPS, min(1 - SMOOTH_ODDS_EPS, odd)) for odd in self.no_odds]
-        return self
-
     @property
     def has_odds(self) -> bool:
         """Check if the problem has odds data."""
-        return self.odds is not None and len(self.odds) > 0
+        odds = self.forecasts[0].odds
+        return odds is not None and len(odds) > 0
 
     @property
     def has_no_odds(self) -> bool:
         """Check if the problem has no_odds data."""
-        return self.no_odds is not None and len(self.no_odds) > 0
+        no_odds = self.forecasts[0].no_odds
+        return no_odds is not None and len(no_odds) > 0
     
     @cached_property
     def crowd_probs(self) -> List[float]:
@@ -153,22 +175,6 @@ class ForecastProblem(BaseModel):
         """Get list of unique forecasters for this problem."""
         return list(set(forecast.username for forecast in self.forecasts))
 
-    @cached_property
-    def option_payoffs(self) -> List[Tuple[int, float]]:
-        """
-        Obtain a sorted list of (option_idx, payoff) for this problem.
-        The payoff is the (1 / odds) for a correct option, and 0 for an incorrect option.
-        """
-        if not self.has_odds:
-            return []
-        
-        option_payoffs = []
-        for option_idx, odd in enumerate(self.odds):
-            payoff = 1 / odd if option_idx in self.correct_option_idx else 0
-            option_payoffs.append((option_idx, payoff))
-        
-        return sorted(option_payoffs, key=lambda x: x[1], reverse=True)
-
 
 class ForecastChallenge(BaseModel):
     """
@@ -223,11 +229,6 @@ class ForecastChallenge(BaseModel):
         """List of unique forecaster usernames."""
         return list(self.forecaster_map.keys())
 
-    @cached_property
-    def problem_option_payoffs(self) -> Dict[str, List[Tuple[int, float]]]:
-        """Map from problem_id to a sorted list of (option_idx, payoff) for this problem."""
-        return {problem.problem_id: problem.option_payoffs for problem in self.forecast_problems}
-
     def get_forecaster_problems(self, username: str) -> List[ForecastProblem]:
         """Get all problems that a specific forecaster participated in."""
         forecaster_problem_ids = {
@@ -346,16 +347,6 @@ class ForecastChallenge(BaseModel):
                 # Next loop: window starts after *upper_boundary*.
                 prev_boundary = upper_boundary
 
-    def fill_problem_with_fair_odds(self, force: bool = False) -> None:
-        """
-        Certain challenge do not have odds data, we can fill in fair/uniform odds for each problem.
-        If `force` is True, we will not check whether the problem already has odds data.
-        """
-        for problem in self.forecast_problems:
-            if problem.has_odds and not force:
-                continue
-            problem.odds = [1 / len(problem.options)] * len(problem.options)
-
 class ChallengeLoader(ABC):
     """
     Abstract base class for loading forecast challenges from different data sources.
