@@ -5,9 +5,11 @@ import logging
 from pathlib import Path
 from typing import Dict, Any, Tuple, List, Optional
 import pandas as pd
-from .base import ChallengeLoader, ForecastChallenge, ForecastProblem, ForecastEvent
+from .base import ChallengeLoader, ForecastChallenge, ForecastProblem, ForecastEvent, ProphetArenaForecastEvent
 from datetime import datetime
 import math
+import numpy as np
+
 from .utils import parse_json_or_eval, get_logger
 
 
@@ -220,25 +222,24 @@ class ProphetArenaChallengeLoader(ChallengeLoader):
         for opt in options:
             info = market_info.get(opt, {})
             yes_ask = info.get(ask_str, None)
-            if info.get('liquidity', None) is not None and info.get('liquidity', None) < 100:
-                asks.append(100)
-            elif yes_ask is not None and yes_ask > 0:
-                if use_bid_for_odds:
-                    if bid_str in info:
-                        yes_bid = info[bid_str]
-                        asks.append((yes_bid + yes_ask) / 2)
-                        # asks.append(max(yes_bid, yes_ask))
-                    else:
-                        asks.append(yes_ask)
-                else:
-                    asks.append(yes_ask)
-            else:
+
+            if yes_ask is None:
                 warning_msg = f"Warning: Option {opt} in market {market_info.get('title', 'Unknown Market')} does not have odds info"
                 logger.warning(warning_msg) if logger is not None else print(
                     warning_msg)
-                asks.append(None)
+                asks.append(100)
+                continue
 
-        implied_probs = [(a / 100.0) if a is not None else 1e-3 for a in asks]
+            if info.get('liquidity', 0) < 100 or yes_ask <= 0:
+                asks.append(100)
+            else:
+                if use_bid_for_odds and bid_str in info:
+                    yes_bid = info[bid_str]
+                    asks.append((yes_bid + yes_ask) / 2)
+                else:
+                    asks.append(yes_ask)
+
+        implied_probs = [(a / 100.0) for a in asks]
         return implied_probs
 
     @staticmethod
@@ -277,12 +278,12 @@ class ProphetArenaChallengeLoader(ChallengeLoader):
         for event_ticker, group in grouped:
             first_row = group.iloc[0]
             problem_id = str(event_ticker)
-            market_info = parse_json_or_eval(
+            first_market_info = parse_json_or_eval(
                 first_row['market_info'], expect_type=dict)
             # skip this market if the market_info is empty
-            if not market_info:
+            if not first_market_info:
                 continue
-            first_option_info = next(iter(market_info.values()))
+            first_option_info = next(iter(first_market_info.values()))
             title = first_option_info.get('title', event_ticker)
 
             if self.use_open_time:
@@ -297,58 +298,84 @@ class ProphetArenaChallengeLoader(ChallengeLoader):
                 end_time = datetime.fromisoformat(close_time.replace(
                     'Z', '+00:00')) if close_time else datetime.now()
             
-            problem_option_keys = list(market_info.keys())
-
-            odds = self._calculate_implied_probs_for_problem(
-                market_info, problem_option_keys, self.use_bid_for_odds, True, self.logger)
-
-            no_odds = self._calculate_implied_probs_for_problem(
-                market_info, problem_option_keys, self.use_bid_for_odds, False, self.logger)
+            # problem_option_keys = literal_eval(first_row['markets']) TODO: need to fix this later
+            problem_option_keys = list(first_market_info.keys())
 
             market_outcome = parse_json_or_eval(
                 first_row['market_outcome'], expect_type=dict)
 
-            correct_option_idx = [i for i, key in enumerate(problem_option_keys) if market_outcome[key] == 1]
+            correct_option_idx = [i for i, key in enumerate(problem_option_keys) if market_outcome.get(key, 0) == 1]
+
             timestamp = datetime.now()
 
             forecasts = []
 
-            if add_market_baseline:
-                # we will add a "market row" in this group, with (unnormalized) prob being simply the market odds
-                forecasts.append(ForecastEvent(
-                    forecast_id=f"{problem_id}-market-baseline",
-                    problem_id=problem_id,
-                    username="market-baseline",
-                    timestamp=timestamp,
-                    probs=self._get_normalized_probs(odds),
-                    unnormalized_probs=odds,
-                ))
-
             category = first_row.get('category', None)
             if category is not None and category not in categories:
                 categories.append(category)
 
-            for _, row in group.iterrows():
+            total_odds, total_no_odds = [], []
+            for i, row in group.iterrows():
+                market_info = parse_json_or_eval(
+                    row['market_info'], expect_type=dict)
+                if not market_info:
+                    continue
+
+                # we calculate the odds/no_odds for each forecast
+                odds = self._calculate_implied_probs_for_problem(
+                    market_info, problem_option_keys, self.use_bid_for_odds, True, self.logger)
+    
+                no_odds = self._calculate_implied_probs_for_problem(
+                    market_info, problem_option_keys, self.use_bid_for_odds, False, self.logger)
+
+                total_odds.append(odds)
+                total_no_odds.append(no_odds)
+
                 username = str(row['predictor_name'])
                 prediction: dict = parse_json_or_eval(
                     row['prediction'], expect_type=dict)
                 probs_dict = {d['market']: d['probability']
                               for d in prediction.get('probabilities', [])}
-                unnormalized_probs = [probs_dict.get(opt, 0.0) for opt in problem_option_keys]
+
+                # clip raw prob to be between 0 and 1
+                unnormalized_probs = [max(0.0, min(1.0, probs_dict.get(opt, 0.0))) for opt in problem_option_keys]
                 # make sure the probs sum to 1
                 probs = self._get_normalized_probs(unnormalized_probs)
 
                 # set `forecast_id` to be `prediction_id` if the column exists, otherwise construct one
                 # using `username` and `problem_id`
-                forecast_id = str(row['prediction_id']) if 'prediction_id' in row else f"{username}-{problem_id}"
+                forecast_id = str(row['prediction_id']) if 'prediction_id' in row else f"{username}-{problem_id}-{i}"
+                submission_id = str(row['submission_id']) if 'submission_id' in row else problem_id
 
-                forecasts.append(ForecastEvent(
+                forecasts.append(ProphetArenaForecastEvent(
                     forecast_id=forecast_id,
                     problem_id=problem_id,
+                    submission_id=submission_id,
                     username=username,
                     timestamp=timestamp,
                     probs=probs,
                     unnormalized_probs=unnormalized_probs,
+                    odds=odds,
+                    no_odds=no_odds,
+                ))
+
+            if add_market_baseline:
+                # we set the submission_id to be the submission_id of the first row in the group
+                submission_id = str(first_row['submission_id'])
+                # use the average odds/no_odds for the market baseline
+                avg_odds = np.mean(total_odds, axis=0).tolist()
+                avg_no_odds = np.mean(total_no_odds, axis=0).tolist()
+                # we will add a "market row" in this group, with (unnormalized) prob being simply the market odds
+                forecasts.append(ProphetArenaForecastEvent(
+                    forecast_id=f"{problem_id}-market-baseline",
+                    problem_id=problem_id,
+                    submission_id=submission_id,
+                    username="market-baseline",
+                    timestamp=timestamp,
+                    probs=self._get_normalized_probs(avg_odds),
+                    unnormalized_probs=avg_odds,
+                    odds=avg_odds,
+                    no_odds=avg_no_odds,
                 ))
 
             if len(forecasts) > 0:
@@ -361,8 +388,6 @@ class ProphetArenaChallengeLoader(ChallengeLoader):
                     end_time=end_time,
                     num_forecasters=len(forecasts),
                     url=None,
-                    odds=odds,
-                    no_odds=no_odds,
                     category=category
                 ))
 
